#!/usr/bin/env python

'''
store calibration data for forces
dictionary with key =	"normal": 4 by a+1 array for normal force,
						"shear"	: 2 by b+1 array for shear force
a = order of normal fit, which is 3
b = order of shear fit, which is 2
'''

import json

import numpy as np
'''
calibration names DO NOT include .json
eg. "COM2" : "original"
'''
sensor_calibrations = {
	"original" : "original", # special case for testing
	"/dev/tty.usbserial-A6WMSVHL" : "original",
	"/dev/tty.usbserial-A8WERMKN" : "original",
	"/dev/tty.usbserial-A6WMNFNP" : "original",
	"/dev/tty.usbserial-A5VAF3WT" : "original", # demo board
	"/dev/tty.usbserial-A6YW6Q3C" : "sensor2_me112",
	"COM22" : "original",
	"COM24" : "original",
	"COM26" : "original",
	"COM4" : "sensor2_4", # Sensor 2 at JPL
	"COM10" : "sensor2_me112"
}

normal_num = 4
normal_fit_order = 3
shear_num = 2
shear_fit_order = 2



CalMatrix = [[	4.3280E-02	,	-4.6509E-03	,	-9.7597E-03	,	6.8750E-03	,	1.2237E-02	,	1.4709E-04	,	5.3329E-05	,	-8.7377E-04	,	-1.2142E-03	,	1.8208E-06	,	-1.8536E-06	,	-4.6660E-06	,	8.8380E-06	,	2.3245E-08	,	-3.5360E-07	,	1.9049E-06	,	4.0616E-06	,	9.8191E-09	,	5.0823E-09	,	-2.1216E-08	,	-4.8366E-08	,	-4.0299E-10	,	9.3209E-11	,	-4.0187E-09	,	-2.8839E-09	],
[	7.1779E-02	,	-1.2751E-02	,	-9.8210E-03	,	1.5086E-02	,	1.4197E-02	,	4.7306E-04	,	-1.3525E-04	,	-5.2301E-04	,	-2.2514E-03	,	8.5894E-07	,	-1.6603E-06	,	-8.4118E-06	,	1.6131E-05	,	-5.4679E-08	,	-4.4024E-07	,	2.8550E-06	,	6.2707E-06	,	1.5555E-08	,	6.1414E-09	,	-3.7148E-08	,	-8.4046E-08	,	-5.8965E-10	,	1.2650E-10	,	-6.4039E-09	,	-4.1437E-09	],
[	-2.5970E-02	,	7.0303E-03	,	5.5552E-03	,	-7.1679E-03	,	-7.2087E-03	,	-7.3321E-03	,	5.7733E-04	,	-1.2310E-02	,	-1.2260E-03	,	-2.1881E-06	,	1.3374E-06	,	5.6331E-07	,	-4.8410E-06	,	4.3624E-07	,	-8.5727E-07	,	1.3590E-06	,	-6.9758E-06	,	3.6617E-09	,	7.0747E-09	,	1.1448E-08	,	1.7833E-08	,	2.9034E-10	,	8.4962E-11	,	1.8585E-09	,	3.1722E-09	],
[	-2.6463E-03	,	3.3018E-04	,	2.5100E-04	,	-4.2063E-04	,	-4.0094E-04	,	2.2740E-05	,	1.4812E-05	,	-7.8569E-05	,	2.1071E-04	,	-7.7002E-08	,	7.1837E-08	,	1.7515E-07	,	-3.7658E-07	,	1.6021E-08	,	2.3714E-09	,	-3.1375E-08	,	-2.6039E-07	,	-2.3046E-10	,	1.9471E-11	,	8.8141E-10	,	1.9060E-09	,	1.4432E-11	,	-1.7694E-12	,	1.5417E-10	,	1.7467E-10	],
[	1.6195E-03	,	-1.9686E-04	,	-3.1875E-04	,	2.5513E-04	,	4.1132E-04	,	-2.3237E-05	,	-8.2600E-05	,	1.5019E-04	,	-6.4464E-05	,	9.6376E-08	,	-1.0768E-07	,	-1.4451E-07	,	2.9382E-07	,	-1.4215E-08	,	4.8830E-10	,	1.7711E-08	,	2.3406E-07	,	2.0528E-10	,	4.0630E-11	,	-6.4583E-10	,	-1.4666E-09	,	-1.4277E-11	,	9.0804E-13	,	-1.7122E-10	,	-1.5544E-10	]];


def EmptyCalibration(name):
	sensor = {}

	sensor["name"] = name

	normal_fit = [0, 0, 0, 0]#[0] * (normal_fit_order + 1)
	normal_arr = [0, 0, 0, 0]#[0] * normal_num
	for i in range(normal_num):
		normal_arr[i] = normal_fit

	sensor["normal"] = normal_arr

	shear_fit = [0, 0, 0] #[0] * (shear_fit_order + 1)
	shear_arr = [0, 0] #[0] * shear_num
	for i in range(shear_num):
		shear_arr[i] = shear_fit

	sensor["shear"] = shear_arr

	return sensor

def LoadCalibration(dev_name):
	print("here")

	cal_name = sensor_calibrations[dev_name]
	filename = cal_name + ".json"

	with open(filename, "r") as f:
		cal = json.load(f)

	if cal["name"] != cal_name:
		raise Exception("Calibration name does not match")

	return cal

def SaveCalibration(name, cal=None):
	if not cal:
		cal = EmptyCalibration(name)

	filename = name + ".json"

	json_cal = json.dumps(cal, sort_keys=True, indent=4, separators=(',', ': '))
	with open(filename, "w") as f:
		f.write(json_cal)

''' x is a scalar and fit is a vector of coefficients '''
def polyfit(x, fit):
	sum = 0
	for i in range(len(fit)):
		sum += pow(x,i) * fit[i]
	return sum

'''
assumes indata is in the format:
[Sx1, Sx2, Sy1, Sy2, T1, T2, T3, T4]

outputs in the format:
[Sx, Sy, T1, T2, T3, T4]
'''
def CalculateForce(indata, calibration):

	data = list(indata)

	if len(data) != 8:
		raise IndexError("data only has {0} of 8 elements".format(len(data)))

	shear_x = [data[0], data[1]]
 	shear_y = [data[2], data[3]]
	normal = data[4:]



	for i in range(len(normal)):
		normal[i] = polyfit(normal[i], calibration["normal"][i])

	for i in range(len(shear)):
		shear[i] = polyfit(shear[i], calibration["shear"][i])

	return shear + normal


#orginal = {
#	"name" : "original",
#	"normal" :	[[-0.00553099, 0.000339965, 2.74158e-08, 8.11113e-13],
#				 [-0.00553099, 0.000339965, 2.74158e-08, 8.11113e-13],
#				 [-0.00553099, 0.000339965, 2.74158e-08, 8.11113e-13],
#				 [-0.00553099, 0.000339965, 2.74158e-08, 8.11113e-13]],
#	"shear" :	[[-0.057098, -0.0062031],
#				 [-0.00058174, -0.00012549]]
#}
#
#
#
#if __name__ == '__main__':
#
#	# SaveCalibration("empty")
#	# SaveCalibration("original", orginal)
#
#	cal = LoadCalibration("COM22")
#
#	# print cal
#
#	print cal["normal"]

